unit GrCtrls;

(*
//.CAPTION Special Graphic Controls
//
//.AUTOR Dmitry M.Ivlev
//.COPYRIGHT DiSoft 1998.
//.VERSION 1.0
//
//.SHORT Unit consists TScaleLine control
//.DESC
//. The TScaleLine control is used to show scale inside graphic window.
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TScaleLine = class(TGraphicControl)
  private
    fScale: Single;
    fNumber: SmallInt;
    fBlackWhite: Boolean;
    procedure SetScale(Value: Single);
    procedure SetNumber(Value: SmallInt);
    procedure SetBlackWhite(Value: Boolean);
  protected
    procedure Paint; override;
  public
    constructor Create(aOwner: TComponent); override;
    procedure UpdateWidth;
  published
    property Scale: Single Read fScale Write SetScale;
    property Number: SmallInt Read fNumber Write SetNumber Default 3;
    property BlackWhite: Boolean Read fBlackWhite Write SetBlackWhite Default FALSE;
    property Font;
    property Hint;
    property ParentColor;
    property ShowHint;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnDragOver;
    property OnDragDrop;
    property OnClick;
  end;

procedure Register;

implementation

{$R *.res}

var
  mmRes, scRes: SmallInt;

constructor TScaleLine.Create(aOwner: TComponent);
begin
  inherited;
  fScale := 1;
  fNumber := 3;
  Height := 20;
  UpdateWidth;
end;

procedure TScaleLine.UpdateWidth;
begin
  Width := fNumber * 100 * scRes div mmRes;
end;

procedure TScaleLine.Paint;
var
  n, h, w: Integer;
  r, cr: TRect;
  e: TSize;
  s: String;
  sc: Single;
begin
  with canvas do begin
    cr := ClientRect;
    SetRect(r, 0, 0, 100 * scRes div mmRes, 15 * scRes div mmRes);
    OffsetRect(r, 0, cr.Bottom - r.Bottom - 1);
    for n := 0 to fNumber-1 do begin
      if (n mod 2 = 0) xor fBlackWhite
      then Brush.Color := clWhite
      else Brush.Color := clBlack;
      Rectangle(r.Left, r.Top, r.Right, r.Bottom);
      OffsetRect(r, 100 * scRes div mmRes, 0);
    end;

    if fScale < 1E-3 then begin h := Round(fScale * 1E+6); s := 'mkm' end else
    if fScale < 1E+0 then begin h := Round(fScale * 1E+3); s := 'mm' end else
    if fScale < 1E+3 then begin h := Round(fScale * 1E+0); s := 'm' end else
                          begin h := Round(fScale * 1E-3); s := 'km'; end;

    s := IntToStr(h)+s;
    e := TextExtent(s);
    SetBkMode(Handle, TRANSPARENT);
    TextOut(cr.Left + (r.Left - cr.Left - e.cx) div 2, r.Top - 1 - e.cy, s);
  end;
end;

procedure TScaleLine.SetScale(Value: Single);
begin
  if fScale = Value then Exit;
  fScale := Value;
  Invalidate;
end;

procedure TScaleLine.SetNumber(Value: SmallInt);
begin
  if fNumber = Value then Exit;
  fNumber := Value;
  UpdateWidth;
  Invalidate;
end;

procedure TScaleLine.SetBlackWhite(Value: Boolean);
begin
  if fBlackWhite = Value then Exit;
  fBlackWhite := Value;
  Invalidate;
end;

procedure Register;
begin
  RegisterComponents('DiSoft', [TScaleLine]);
end;

procedure GetResolution;
var
  dc: HDC;
  sz: TSize;
begin
  dc := GetDC(0);
  SetMapMode(dc, MM_LOMETRIC);
  GetViewportExtEx(dc, sz); scRes := sz.cx;
  GetWindowExtEx(dc, sz); mmRes := sz.cx;
  ReleaseDC(0, dc);
end;

initialization
  GetResolution;
end.
